@@ -124,6 +124,7 @@ class SvcHub(object):
         self.argv = argv
         self.E: EnvParams = args.E
         self.no_ansi = args.no_ansi
+        self.flo = args.flo
         self.tz = UTC if args.log_utc else None
         self.logf: Optional[typing.TextIO] = None
         self.logf_base_fn = ""
@@ -220,6 +221,10 @@ class SvcHub(object):
             self.log("root", t.format(args.j), c=3)
             args.no_fpool = True
 
+        args.p_nodav = [int(x.strip()) for x in args.p_nodav.split(",") if x]
+        if args.dav_port and args.dav_port not in args.p:
+            args.p.append(args.dav_port)
+
         for name, arg in (
             ("iobuf", "iobuf"),
             ("s-rd-sz", "s_rd_sz"),
@@ -311,6 +316,10 @@ class SvcHub(object):
         args.doctitle = args.doctitle.replace("--name", args.vname)
         args.bname = args.bname.replace("--name", args.vname) or args.vname
 
+        for zs in "shr_site up_site".split():
+            if getattr(args, zs) == "--site":
+                setattr(args, zs, args.site)
+
         if args.log_fk:
             args.log_fk = re.compile(args.log_fk)
 
@@ -409,6 +418,11 @@ class SvcHub(object):
         if not args.http_only:
             zms += "D"
 
+        if args.sftp:
+            from .sftpd import Sftpd
+
+            self.sftpd: Optional[Sftpd] = None
+
         if args.ftp or args.ftps:
             from .ftpd import Ftpd
 
@@ -420,7 +434,7 @@ class SvcHub(object):
 
             self.tftpd: Optional[Tftpd] = None
 
-        if args.ftp or args.ftps or args.tftp:
+        if args.sftp or args.ftp or args.ftps or args.tftp:
             Daemon(self.start_ftpd, "start_tftpd")
 
         if args.smb:
@@ -747,12 +761,28 @@ class SvcHub(object):
     def start_ftpd(self) -> None:
         time.sleep(30)
 
+        if hasattr(self, "sftpd") and not self.sftpd:
+            self.restart_sftpd()
+
         if hasattr(self, "ftpd") and not self.ftpd:
             self.restart_ftpd()
 
         if hasattr(self, "tftpd") and not self.tftpd:
             self.restart_tftpd()
 
+    def restart_sftpd(self) -> None:
+        if not hasattr(self, "sftpd"):
+            return
+
+        from .sftpd import Sftpd
+
+        if self.sftpd:
+            return  # todo
+
+        self.sftpd = Sftpd(self)
+        self.sftpd.run()
+        self.log("root", "started SFTPd")
+
     def restart_ftpd(self) -> None:
         if not hasattr(self, "ftpd"):
             return
@@ -889,9 +919,9 @@ class SvcHub(object):
             return
 
         ar = self.args
-        for _ in range(10 if ar.ftp or ar.ftps else 0):
+        for _ in range(10 if ar.sftp or ar.ftp or ar.ftps else 0):
             time.sleep(0.03)
-            if self.ftpd:
+            if self.ftpd if ar.ftp or ar.ftps else ar.sftp:
                 break
 
         if self.tcpsrv.qr:
@@ -1088,6 +1118,12 @@ class SvcHub(object):
             vsa = [x.lower() for x in vsa if x]
             setattr(al, k + "_set", set(vsa))
 
+        for k in "smsg".split(" "):
+            vs = getattr(al, k)
+            vsa = [x.strip() for x in vs.split(",")]
+            vsa = [x.upper() for x in vsa if x]
+            setattr(al, k + "_set", set(vsa))
+
         zs = "dav_ua1 sus_urls nonsus_urls ua_nodav ua_nodoc ua_nozip"
         for k in zs.split(" "):
             vs = getattr(al, k)
@@ -1143,9 +1179,15 @@ class SvcHub(object):
             zs = zs[3:]
         al.idp_chsub_tr = umktrans(zs1, zs2)
 
+        al.sftp_ipa_nm = build_netmap(al.sftp_ipa or al.ipa or al.ipar, True)
         al.ftp_ipa_nm = build_netmap(al.ftp_ipa or al.ipa or al.ipar, True)
         al.tftp_ipa_nm = build_netmap(al.tftp_ipa or al.ipa or al.ipar, True)
 
+        al.sftp_key2u = {
+            "%s %s" % (x[1], x[2]): x[0]
+            for x in [x.split(" ") for x in al.sftp_key or []]
+        }
+
         mte = ODict.fromkeys(DEF_MTE.split(","), True)
         al.mte = odfusion(mte, al.mte)
 
@@ -1536,21 +1578,38 @@ class SvcHub(object):
                 dt.microsecond // self.log_div,
             )
 
-            if c and not self.args.no_ansi:
-                if isinstance(c, int):
+            if self.flo == 1:
+                fmt = "@%s [%-21s] %s\n"
+                if not c:
+                    if "\033" in msg:
+                        msg += "\033[0m"
+                elif isinstance(c, int):
                     msg = "\033[3%sm%s\033[0m" % (c, msg)
                 elif "\033" not in c:
                     msg = "\033[%sm%s\033[0m" % (c, msg)
                 else:
                     msg = "%s%s\033[0m" % (c, msg)
 
-            if "\033" in src:
-                src += "\033[0m"
+                if "\033" in src:
+                    src += "\033[0m"
+            else:
+                if not c:
+                    fmt = "@%s  LOG [%-21s] %s\n"
+                elif c == 1:
+                    fmt = "@%s CRIT [%-21s] %s\n"
+                elif c == 3:
+                    fmt = "@%s WARN [%-21s] %s\n"
+                elif c == 6:
+                    fmt = "@%s  BTW [%-21s] %s\n"
+                else:
+                    fmt = "@%s  LOG [%-21s] %s\n"
 
-            if "\033" in msg:
-                msg += "\033[0m"
+                if "\033" in src:
+                    src = RE_ANSI.sub("", src)
+                if "\033" in msg:
+                    msg = RE_ANSI.sub("", msg)
 
-            self.logf.write("@%s [%-21s] %s\n" % (ts, src, msg))
+            self.logf.write(fmt % (ts, src, msg))
             if not self.args.no_logflush:
                 self.logf.flush()
 
@@ -1580,9 +1639,10 @@ class SvcHub(object):
                 if self.logf:
                     self.logf.write(zs)
 
-            fmt = "\033[36m%s \033[33m%-21s \033[0m%s\n"
             if self.no_ansi:
-                if c == 1:
+                if not c:
+                    fmt = "%s %-21s  LOG: %s\n"
+                elif c == 1:
                     fmt = "%s %-21s CRIT: %s\n"
                 elif c == 3:
                     fmt = "%s %-21s WARN: %s\n"
@@ -1590,12 +1650,16 @@ class SvcHub(object):
                     fmt = "%s %-21s  BTW: %s\n"
                 else:
                     fmt = "%s %-21s  LOG: %s\n"
+
                 if "\033" in msg:
                     msg = RE_ANSI.sub("", msg)
                 if "\033" in src:
                     src = RE_ANSI.sub("", src)
-            elif c:
-                if isinstance(c, int):
+            else:
+                fmt = "\033[36m%s \033[33m%-21s \033[0m%s\n"
+                if not c:
+                    pass
+                elif isinstance(c, int):
                     msg = "\033[3%sm%s\033[0m" % (c, msg)
                 elif "\033" not in c:
                     msg = "\033[%sm%s\033[0m" % (c, msg)
